package backend

import (
	"encoding/json"
	"errors"
	"fmt"

	"git.faercol.me/faercol/polyculeconnect/polyculeconnect/connector"
	"github.com/dexidp/dex/connector/oidc"
	"github.com/dexidp/dex/storage"
)

var ErrUnsupportedType = errors.New("unsupported connector type")

type BackendConfig struct {
	OIDC    oidc.Config
	Storage storage.Connector
}

func (bc *BackendConfig) FromConnector(connector storage.Connector) error {
	if connector.Type != "oidc" {
		return ErrUnsupportedType
	}
	if err := json.Unmarshal(connector.Config, &bc.OIDC); err != nil {
		return fmt.Errorf("invalid OIDC config: %w", err)
	}
	bc.Storage = connector
	return nil
}

type Service interface {
	ListBackends() ([]BackendConfig, error)
	GetBackend(id string) (BackendConfig, error)
	AddBackend(config BackendConfig) error
	RemoveBackend(id string) error
}

type concreteBackendService struct {
	s storage.Storage
}

func (cbs *concreteBackendService) ListBackends() ([]BackendConfig, error) {
	connectors, err := cbs.s.ListConnectors()
	if err != nil {
		return nil, fmt.Errorf("failed to get connectors from storage: %w", err)
	}
	var res []BackendConfig
	for _, c := range connectors {

		// We know that this type is special, we don't want to use it at all here
		if c.Type == connector.TypeRefuseAll {
			continue
		}

		var b BackendConfig
		if err := b.FromConnector(c); err != nil {
			return res, err
		}
		res = append(res, b)
	}
	return res, nil
}

func (cbs *concreteBackendService) GetBackend(connectorID string) (BackendConfig, error) {
	c, err := cbs.s.GetConnector(connectorID)
	if err != nil {
		return BackendConfig{}, fmt.Errorf("failed to get connector from storage: %w", err)
	}
	var res BackendConfig
	if err := res.FromConnector(c); err != nil {
		return BackendConfig{}, err
	}
	return res, nil
}

func (cbs *concreteBackendService) AddBackend(config BackendConfig) error {
	return cbs.s.CreateConnector(config.Storage)
}

func (cbs *concreteBackendService) RemoveBackend(connectorID string) error {
	return cbs.s.DeleteConnector(connectorID)
}

func New(s storage.Storage) Service {
	return &concreteBackendService{s}
}
